#!/usr/bin/env python
from .command import Command
from .deploy_ticket import execute_ticket
from matador.session import Session
import subprocess
import os
import shutil
import yaml
from importlib.machinery import SourceFileLoader


class ActionPackage(Command):

    def _add_arguments(self, parser):
        parser.prog = 'matador deploy-package'
        parser.add_argument(
            '-e', '--environment',
            type=str,
            required=True,
            help='Agresso environment name')

        parser.add_argument(
            '-p', '--package',
            type=str,
            required=True,
            help='Package name')

        parser.add_argument(
            '-c', '--commit',
            type=str,
            default='none',
            help='Commit or tag ID')

    @staticmethod
    def _checkout_package(package, commit):
        proj_folder = Session.project_folder
        repo_folder = Session.matador_repository_folder
        package_folder = os.path.join(
            Session.matador_packages_folder, package)

        Session.update_repository()

        if commit == 'none':
            commit = subprocess.check_output(
                ['git', '-C', proj_folder, 'rev-parse', 'HEAD'],
                stderr=subprocess.STDOUT).decode('utf-8').strip('\n')

        subprocess.run([
            'git', '-C', repo_folder, 'checkout', commit],
            stderr=subprocess.STDOUT,
            stdout=open(os.devnull, 'w'),
            check=True)

        src = os.path.join(repo_folder, 'deploy', 'packages', package)
        shutil.copytree(src, package_folder)

    def _execute(self):
        Session.set_environment(self.args.environment)
        self._checkout_package(self.args.package, self.args.commit)


class DeployPackage(ActionPackage):

    def _execute(self):
        super(DeployPackage, self)._execute()
        package_folder = os.path.join(
            Session.matador_packages_folder, self.args.package)
        Session.deployment_folder = package_folder
        ticketsFile = os.path.join(package_folder, 'tickets.yml')
        try:
            tickets = yaml.load(open(ticketsFile, 'r'))
            for ticket in tickets:
                execute_ticket(str(ticket), 'deploy', self.args.commit, True)
        finally:
            shutil.rmtree(package_folder)


class RemovePackage(ActionPackage):

    def _execute(self):
        super(RemovePackage, self)._execute()
        package_folder = os.path.join(
            Session.matador_packages_folder, self.args.package)
        Session.deployment_folder = package_folder
        sourceFile = os.path.join(package_folder, 'remove.py')
        try:
            SourceFileLoader('remove', sourceFile).load_module()
        finally:
            shutil.rmtree(package_folder)



#!/usr/bin/env python
from .command import Command
from matador.session import Session
import os
import subprocess


def add_to_git(directory, message):
    subprocess.run([
        'git', '-C', directory, 'add', '-A'],
        stderr=subprocess.STDOUT,
        stdout=open(os.devnull, 'w'))

    subprocess.run([
        'git', '-C', directory, 'commit', '-m', message],
        stderr=subprocess.STDOUT,
        stdout=open(os.devnull, 'w'))


class CreateTicket(Command):

    def _add_arguments(self, parser):
        parser.prog = 'matador create-ticket'

        parser.add_argument(
            '-t', '--ticket',
            type=str,
            required=True,
            help='Ticket name')

    def _execute(self):
        ticket_folder = os.path.join(
            Session.project_folder, 'deploy', 'tickets', self.args.ticket)
        os.makedirs(ticket_folder)
        deploy_file = os.path.join(ticket_folder, 'deploy.py')

        with open(deploy_file, 'w') as f:
            f.write('from matador.commands.deployment import *\n\n')
            f.close()

        add_to_git(
            Session.project_folder, 'Create ticket %s' % self.args.ticket)


class CreatePackage(Command):

    def _add_arguments(self, parser):
        parser.prog = 'matador create-package'

        parser.add_argument(
            '-p', '--package',
            type=str,
            required=True,
            help='Ticket name')

    def _execute(self):
        package_folder = os.path.join(
            Session.project_folder, 'deploy', 'packages', self.args.package)
        os.makedirs(package_folder)

        package_file = os.path.join(package_folder, 'tickets.yml')
        with open(package_file, 'w') as f:
            f.write(
                '# List each ticket on a separate line preceded by - . e.g.\n')
            f.write('# - 30\n')
            f.write('# - 31\n')
            f.close()

        remove_file = os.path.join(package_folder, 'remove.py')
        with open(remove_file, 'w') as f:
            f.write('from matador.commands.deployment import *\n\n')
            f.close()

        add_to_git(
            Session.project_folder, 'Create package %s' % self.args.package)


class AddTicketToPackage(Command):

    def _add_arguments(self, parser):
        parser.prog = 'matador add-t2p'

        parser.add_argument(
            '-t', '--ticket',
            type=str,
            required=True,
            help='Ticket name')

        parser.add_argument(
            '-p', '--package',
            type=str,
            required=True,
            help='Ticket name')

    def _execute(self):
        package_file = os.path.join(
            Session.project_folder, 'deploy', 'packages', self.args.package,
            'tickets.yml')

        with open(package_file, 'a') as f:
            f.write('- %s\n' % self.args.ticket)
            f.close()

        add_to_git(
            Session.project_folder,
            'Add ticket %s to package %s' % (
                self.args.ticket, self.args.package))


